@@ -20,6 +20,7 @@
 #pragma once
 #include "vulkan/vulkan.h"
 #include "vulkan/vk_layer.h"
+#include <string>
 #include <unordered_map>
 #include <stdbool.h>
 #include <stdio.h>
@@ -44,7 +45,7 @@ const std::unordered_map<std::string, VkFlags> debug_actions_option_definitions
     {std::string("VK_DBG_LAYER_ACTION_CALLBACK"), VK_DBG_LAYER_ACTION_CALLBACK},
     {std::string("VK_DBG_LAYER_ACTION_LOG_MSG"), VK_DBG_LAYER_ACTION_LOG_MSG},
     {std::string("VK_DBG_LAYER_ACTION_BREAK"), VK_DBG_LAYER_ACTION_BREAK},
-#if WIN32
+#if defined(WIN32)
     {std::string("VK_DBG_LAYER_ACTION_DEBUG_OUTPUT"), VK_DBG_LAYER_ACTION_DEBUG_OUTPUT},
 #endif
     {std::string("VK_DBG_LAYER_ACTION_DEFAULT"), VK_DBG_LAYER_ACTION_DEFAULT}};
