@@ -28,18 +28,17 @@ class VkImageObj;
 #include "vktestframework.h"
 #endif
 
+#include <array>
 #include <map>
 #include <vector>
-#include <array>
 
 using namespace std;
 
 class VkDeviceObj : public vk_testing::Device {
   public:
     VkDeviceObj(uint32_t id, VkPhysicalDevice obj);
-    VkDeviceObj(uint32_t id, VkPhysicalDevice obj,
-                std::vector<const char *> &layers,
-                std::vector<const char *> &extension_names);
+    VkDeviceObj(uint32_t id, VkPhysicalDevice obj, std::vector<const char *> &extension_names,
+                VkPhysicalDeviceFeatures *features = nullptr);
 
     VkDevice device() { return handle(); }
     void get_device_queue();
@@ -71,19 +70,14 @@ class VkRenderFramework : public VkTestFramework {
     void InitRenderTarget(VkImageView *dsBinding);
     void InitRenderTarget(uint32_t targets, VkImageView *dsBinding);
     void InitFramework();
-    void InitFramework(std::vector<const char *> instance_layer_names,
-                       std::vector<const char *> device_layer_names,
-                       std::vector<const char *> instance_extension_names,
-                       std::vector<const char *> device_extension_names,
-                       PFN_vkDebugReportCallbackEXT = NULL,
+    void InitFramework(std::vector<const char *> instance_layer_names, std::vector<const char *> instance_extension_names,
+                       std::vector<const char *> device_extension_names, PFN_vkDebugReportCallbackEXT = NULL,
                        void *userData = NULL);
 
     void ShutdownFramework();
-    void InitState();
+    void InitState(VkPhysicalDeviceFeatures *features = nullptr);
 
-    const VkRenderPassBeginInfo &renderPassBeginInfo() const {
-        return m_renderPassBeginInfo;
-    }
+    const VkRenderPassBeginInfo &renderPassBeginInfo() const { return m_renderPassBeginInfo; }
 
   protected:
     VkApplicationInfo app_info;
@@ -123,6 +117,7 @@ class VkRenderFramework : public VkTestFramework {
     PFN_vkDebugReportMessageEXT m_DebugReportMessage;
     VkDebugReportCallbackEXT m_globalMsgCallback;
     VkDebugReportCallbackEXT m_devMsgCallback;
+    std::vector<const char *> device_extension_names;
 
     /*
      * SetUp and TearDown are called by the Google Test framework
@@ -156,57 +151,45 @@ class VkCommandBufferObj : public vk_testing::CommandBuffer {
     VkResult BeginCommandBuffer();
     VkResult BeginCommandBuffer(VkCommandBufferBeginInfo *pInfo);
     VkResult EndCommandBuffer();
-    void PipelineBarrier(VkPipelineStageFlags src_stages,
-                         VkPipelineStageFlags dest_stages,
-                         VkDependencyFlags dependencyFlags,
-                         uint32_t memoryBarrierCount,
-                         const VkMemoryBarrier *pMemoryBarriers,
-                         uint32_t bufferMemoryBarrierCount,
-                         const VkBufferMemoryBarrier *pBufferMemoryBarriers,
-                         uint32_t imageMemoryBarrierCount,
+    void PipelineBarrier(VkPipelineStageFlags src_stages, VkPipelineStageFlags dest_stages, VkDependencyFlags dependencyFlags,
+                         uint32_t memoryBarrierCount, const VkMemoryBarrier *pMemoryBarriers, uint32_t bufferMemoryBarrierCount,
+                         const VkBufferMemoryBarrier *pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount,
                          const VkImageMemoryBarrier *pImageMemoryBarriers);
     void AddRenderTarget(VkImageObj *renderTarget);
     void AddDepthStencil();
-    void ClearAllBuffers(VkClearColorValue clear_color, float depth_clear_color,
-                         uint32_t stencil_clear_color,
+    void ClearAllBuffers(VkClearColorValue clear_color, float depth_clear_color, uint32_t stencil_clear_color,
                          VkDepthStencilObj *depthStencilObj);
     void PrepareAttachments();
     void BindPipeline(VkPipelineObj &pipeline);
     void BindDescriptorSet(VkDescriptorSetObj &descriptorSet);
-    void BindVertexBuffer(VkConstantBufferObj *vertexBuffer,
-                          VkDeviceSize offset, uint32_t binding);
+    void BindVertexBuffer(VkConstantBufferObj *vertexBuffer, VkDeviceSize offset, uint32_t binding);
     void BindIndexBuffer(VkIndexBufferObj *indexBuffer, VkDeviceSize offset);
     void BeginRenderPass(const VkRenderPassBeginInfo &info);
     void EndRenderPass();
-    void FillBuffer(VkBuffer buffer, VkDeviceSize offset,
-                    VkDeviceSize fill_size, uint32_t data);
-    void Draw(uint32_t vertexCount, uint32_t instanceCount,
-              uint32_t firstVertex, uint32_t firstInstance);
-    void DrawIndexed(uint32_t indexCount, uint32_t instanceCount,
-                     uint32_t firstIndex, int32_t vertexOffset,
+    void FillBuffer(VkBuffer buffer, VkDeviceSize offset, VkDeviceSize fill_size, uint32_t data);
+    void Draw(uint32_t vertexCount, uint32_t instanceCount, uint32_t firstVertex, uint32_t firstInstance);
+    void DrawIndexed(uint32_t indexCount, uint32_t instanceCount, uint32_t firstIndex, int32_t vertexOffset,
                      uint32_t firstInstance);
     void QueueCommandBuffer(bool checkSuccess = true);
     void QueueCommandBuffer(VkFence fence, bool checkSuccess = true);
-    void SetViewport(uint32_t firstViewport, uint32_t viewportCount,
-                     const VkViewport *pViewports);
-    void SetScissor(uint32_t firstScissor, uint32_t scissorCount,
-                    const VkRect2D *pScissors);
+    void SetViewport(uint32_t firstViewport, uint32_t viewportCount, const VkViewport *pViewports);
+    void SetScissor(uint32_t firstScissor, uint32_t scissorCount, const VkRect2D *pScissors);
     void SetLineWidth(float lineWidth);
-    void SetDepthBias(float depthBiasConstantFactor, float depthBiasClamp,
-                      float depthBiasSlopeFactor);
+    void SetDepthBias(float depthBiasConstantFactor, float depthBiasClamp, float depthBiasSlopeFactor);
     void SetBlendConstants(const float blendConstants[4]);
     void SetDepthBounds(float minDepthBounds, float maxDepthBounds);
     void SetStencilReadMask(VkStencilFaceFlags faceMask, uint32_t compareMask);
     void SetStencilWriteMask(VkStencilFaceFlags faceMask, uint32_t writeMask);
     void SetStencilReference(VkStencilFaceFlags faceMask, uint32_t reference);
-    void UpdateBuffer(VkBuffer buffer, VkDeviceSize dstOffset,
-                      VkDeviceSize dataSize, const void *pData);
-    void CopyImage(VkImage srcImage, VkImageLayout srcImageLayout,
-                   VkImage dstImage, VkImageLayout dstImageLayout,
+    void UpdateBuffer(VkBuffer buffer, VkDeviceSize dstOffset, VkDeviceSize dataSize, const void *pData);
+    void CopyImage(VkImage srcImage, VkImageLayout srcImageLayout, VkImage dstImage, VkImageLayout dstImageLayout,
                    uint32_t regionCount, const VkImageCopy *pRegions);
-    void ResolveImage(VkImage srcImage, VkImageLayout srcImageLayout,
-                      VkImage dstImage, VkImageLayout dstImageLayout,
+    void ResolveImage(VkImage srcImage, VkImageLayout srcImageLayout, VkImage dstImage, VkImageLayout dstImageLayout,
                       uint32_t regionCount, const VkImageResolve *pRegions);
+    void ClearColorImage(VkImage image, VkImageLayout imageLayout, const VkClearColorValue *pColor, uint32_t rangeCount,
+                         const VkImageSubresourceRange *pRanges);
+    void ClearDepthStencilImage(VkImage image, VkImageLayout imageLayout, const VkClearDepthStencilValue *pColor,
+                                uint32_t rangeCount, const VkImageSubresourceRange *pRanges);
 
   protected:
     VkDeviceObj *m_device;
@@ -216,33 +199,20 @@ class VkCommandBufferObj : public vk_testing::CommandBuffer {
 class VkConstantBufferObj : public vk_testing::Buffer {
   public:
     VkConstantBufferObj(VkDeviceObj *device,
-                        VkBufferUsageFlags usage =
-            VK_BUFFER_USAGE_TRANSFER_SRC_BIT |
-            VK_BUFFER_USAGE_TRANSFER_DST_BIT);
-    VkConstantBufferObj(VkDeviceObj *device, int constantCount,
-                        int constantSize, const void *data,
-                        VkBufferUsageFlags usage =
-            VK_BUFFER_USAGE_TRANSFER_SRC_BIT |
-            VK_BUFFER_USAGE_TRANSFER_DST_BIT);
+                        VkBufferUsageFlags usage = VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT);
+    VkConstantBufferObj(VkDeviceObj *device, int constantCount, int constantSize, const void *data,
+                        VkBufferUsageFlags usage = VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT);
     ~VkConstantBufferObj();
-    void BufferMemoryBarrier(
-        VkFlags srcAccessMask = VK_ACCESS_HOST_WRITE_BIT |
-                                VK_ACCESS_SHADER_WRITE_BIT |
-                                VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT |
-                                VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT |
-                                VK_ACCESS_TRANSFER_WRITE_BIT,
-        VkFlags dstAccessMask = VK_ACCESS_HOST_READ_BIT |
-                                VK_ACCESS_INDIRECT_COMMAND_READ_BIT |
-                                VK_ACCESS_INDEX_READ_BIT |
-                                VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT |
-                                VK_ACCESS_UNIFORM_READ_BIT |
-                                VK_ACCESS_SHADER_READ_BIT |
-                                VK_ACCESS_COLOR_ATTACHMENT_READ_BIT |
-                                VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT |
-                                VK_ACCESS_MEMORY_READ_BIT);
+    void BufferMemoryBarrier(VkFlags srcAccessMask = VK_ACCESS_HOST_WRITE_BIT | VK_ACCESS_SHADER_WRITE_BIT |
+                                                     VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT |
+                                                     VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT | VK_ACCESS_TRANSFER_WRITE_BIT,
+                             VkFlags dstAccessMask = VK_ACCESS_HOST_READ_BIT | VK_ACCESS_INDIRECT_COMMAND_READ_BIT |
+                                                     VK_ACCESS_INDEX_READ_BIT | VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT |
+                                                     VK_ACCESS_UNIFORM_READ_BIT | VK_ACCESS_SHADER_READ_BIT |
+                                                     VK_ACCESS_COLOR_ATTACHMENT_READ_BIT |
+                                                     VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT | VK_ACCESS_MEMORY_READ_BIT);
 
-    void Bind(VkCommandBuffer commandBuffer, VkDeviceSize offset,
-              uint32_t binding);
+    void Bind(VkCommandBuffer commandBuffer, VkDeviceSize offset, uint32_t binding);
 
     VkDescriptorBufferInfo m_descriptorBufferInfo;
 
@@ -259,8 +229,7 @@ class VkConstantBufferObj : public vk_testing::Buffer {
 class VkIndexBufferObj : public VkConstantBufferObj {
   public:
     VkIndexBufferObj(VkDeviceObj *device);
-    void CreateAndInitBuffer(int numIndexes, VkIndexType dataFormat,
-                             const void *data);
+    void CreateAndInitBuffer(int numIndexes, VkIndexType dataFormat, const void *data);
     void Bind(VkCommandBuffer commandBuffer, VkDeviceSize offset);
     VkIndexType GetIndexType();
 
@@ -268,21 +237,32 @@ class VkIndexBufferObj : public VkConstantBufferObj {
     VkIndexType m_indexType;
 };
 
+class VkRenderpassObj {
+  public:
+    VkRenderpassObj(VkDeviceObj *device);
+    ~VkRenderpassObj();
+    VkRenderPass handle() { return m_renderpass; }
+
+  protected:
+    VkRenderPass m_renderpass;
+    VkDevice device;
+};
+
 class VkImageObj : public vk_testing::Image {
   public:
     VkImageObj(VkDeviceObj *dev);
     bool IsCompatible(VkFlags usage, VkFlags features);
 
   public:
-    void init(uint32_t w, uint32_t h, VkFormat fmt, VkFlags usage,
-              VkImageTiling tiling = VK_IMAGE_TILING_LINEAR,
+    void init(uint32_t w, uint32_t h, VkFormat fmt, VkFlags usage, VkImageTiling tiling = VK_IMAGE_TILING_LINEAR,
               VkMemoryPropertyFlags reqs = 0);
 
+    void init_no_layout(uint32_t w, uint32_t h, VkFormat fmt, VkFlags usage, VkImageTiling tiling = VK_IMAGE_TILING_LINEAR,
+                        VkMemoryPropertyFlags reqs = 0);
+
     //    void clear( CommandBuffer*, uint32_t[4] );
 
-    void layout(VkImageLayout layout) {
-        m_descriptorImageInfo.imageLayout = layout;
-    }
+    void layout(VkImageLayout layout) { m_descriptorImageInfo.imageLayout = layout; }
 
     VkDeviceMemory memory() const { return Image::memory().handle(); }
 
@@ -290,8 +270,7 @@ class VkImageObj : public vk_testing::Image {
 
     void UnmapMemory() { Image::memory().unmap(); }
 
-    void ImageMemoryBarrier(VkCommandBufferObj *cmd, VkImageAspectFlags aspect,
-                            VkFlags output_mask, VkFlags input_mask,
+    void ImageMemoryBarrier(VkCommandBufferObj *cmd, VkImageAspectFlags aspect, VkFlags output_mask, VkFlags input_mask,
                             VkImageLayout image_layout);
 
     VkResult CopyImage(VkImageObj &src_image);
@@ -309,16 +288,14 @@ class VkImageObj : public vk_testing::Image {
             createView.components.g = VK_COMPONENT_SWIZZLE_G;
             createView.components.b = VK_COMPONENT_SWIZZLE_B;
             createView.components.a = VK_COMPONENT_SWIZZLE_A;
-            createView.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0,
-                                           1};
+            createView.subresourceRange = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1};
             createView.flags = 0;
             m_targetView.init(*m_device, createView);
         }
         return m_targetView.handle();
     }
 
-    void SetLayout(VkCommandBufferObj *cmd_buf, VkImageAspectFlags aspect,
-                   VkImageLayout image_layout);
+    void SetLayout(VkCommandBufferObj *cmd_buf, VkImageAspectFlags aspect, VkImageLayout image_layout);
     void SetLayout(VkImageAspectFlags aspect, VkImageLayout image_layout);
 
     VkImageLayout layout() const { return m_descriptorImageInfo.imageLayout; }
@@ -348,8 +325,8 @@ class VkTextureObj : public VkImageObj {
 class VkDepthStencilObj : public VkImageObj {
   public:
     VkDepthStencilObj(VkDeviceObj *device);
-    void Init(VkDeviceObj *device, int32_t width, int32_t height,
-                       VkFormat format, VkImageUsageFlags usage = VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT);
+    void Init(VkDeviceObj *device, int32_t width, int32_t height, VkFormat format,
+              VkImageUsageFlags usage = VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT);
     bool Initialized();
     VkImageView *BindInfo();
 
@@ -375,14 +352,13 @@ class VkDescriptorSetObj : public vk_testing::DescriptorPool {
     ~VkDescriptorSetObj();
 
     int AppendDummy();
-    int AppendBuffer(VkDescriptorType type,
-                     VkConstantBufferObj &constantBuffer);
+    int AppendBuffer(VkDescriptorType type, VkConstantBufferObj &constantBuffer);
     int AppendSamplerTexture(VkSamplerObj *sampler, VkTextureObj *texture);
     void CreateVKDescriptorSet(VkCommandBufferObj *commandBuffer);
 
     VkDescriptorSet GetDescriptorSetHandle() const;
     VkPipelineLayout GetPipelineLayout() const;
-    int GetTypeCounts() {return m_type_counts.size();}
+    int GetTypeCounts() { return m_type_counts.size(); }
 
   protected:
     VkDeviceObj *m_device;
@@ -400,8 +376,7 @@ class VkDescriptorSetObj : public vk_testing::DescriptorPool {
 
 class VkShaderObj : public vk_testing::ShaderModule {
   public:
-    VkShaderObj(VkDeviceObj *device, const char *shaderText,
-                VkShaderStageFlagBits stage, VkRenderFramework *framework,
+    VkShaderObj(VkDeviceObj *device, const char *shaderText, VkShaderStageFlagBits stage, VkRenderFramework *framework,
                 char const *name = "main");
     VkPipelineShaderStageCreateInfo GetStageCreateInfo() const;
 
@@ -416,12 +391,9 @@ class VkPipelineObj : public vk_testing::Pipeline {
   public:
     VkPipelineObj(VkDeviceObj *device);
     void AddShader(VkShaderObj *shaderObj);
-    void AddVertexInputAttribs(VkVertexInputAttributeDescription *vi_attrib,
-                               uint32_t count);
-    void AddVertexInputBindings(VkVertexInputBindingDescription *vi_binding,
-                                uint32_t count);
-    void AddColorAttachment(uint32_t binding,
-                            const VkPipelineColorBlendAttachmentState *att);
+    void AddVertexInputAttribs(VkVertexInputAttributeDescription *vi_attrib, uint32_t count);
+    void AddVertexInputBindings(VkVertexInputBindingDescription *vi_binding, uint32_t count);
+    void AddColorAttachment(uint32_t binding, const VkPipelineColorBlendAttachmentState *att);
     void MakeDynamic(VkDynamicState state);
 
     void AddColorAttachment() {
@@ -438,15 +410,14 @@ class VkPipelineObj : public vk_testing::Pipeline {
     void SetTessellation(const VkPipelineTessellationStateCreateInfo *te_state);
     void SetViewport(const vector<VkViewport> viewports);
     void SetScissor(const vector<VkRect2D> scissors);
-    VkResult CreateVKPipeline(VkPipelineLayout layout,
-                              VkRenderPass render_pass);
+    VkResult CreateVKPipeline(VkPipelineLayout layout, VkRenderPass render_pass);
 
   protected:
     VkPipelineVertexInputStateCreateInfo m_vi_state;
     VkPipelineInputAssemblyStateCreateInfo m_ia_state;
     VkPipelineRasterizationStateCreateInfo m_rs_state;
     VkPipelineColorBlendStateCreateInfo m_cb_state;
-    VkPipelineDepthStencilStateCreateInfo m_ds_state;
+    VkPipelineDepthStencilStateCreateInfo const *m_ds_state;
     VkPipelineViewportStateCreateInfo m_vp_state;
     VkPipelineMultisampleStateCreateInfo m_ms_state;
     VkPipelineTessellationStateCreateInfo m_te_state;
