@@ -19,62 +19,53 @@
  * Author: Tony Barbour <tony@LunarG.com>
  */
 
-#include <iostream>
-#include <string.h> // memset(), memcmp()
-#include <assert.h>
-#include <stdarg.h>
 #include "vktestbinding.h"
+#include <assert.h>
+#include <iostream>
+#include <stdarg.h>
+#include <string.h> // memset(), memcmp()
 
 namespace {
 
-#define NON_DISPATCHABLE_HANDLE_INIT(create_func, dev, ...)                    \
-    do {                                                                       \
-        handle_type handle;                                                    \
-        if (EXPECT(create_func(dev.handle(), __VA_ARGS__, NULL, &handle) ==    \
-                   VK_SUCCESS))                                                \
-            NonDispHandle::init(dev.handle(), handle);                         \
+#define NON_DISPATCHABLE_HANDLE_INIT(create_func, dev, ...)                                                                        \
+    do {                                                                                                                           \
+        handle_type handle;                                                                                                        \
+        if (EXPECT(create_func(dev.handle(), __VA_ARGS__, NULL, &handle) == VK_SUCCESS))                                           \
+            NonDispHandle::init(dev.handle(), handle);                                                                             \
     } while (0)
 
-#define NON_DISPATCHABLE_HANDLE_DTOR(cls, destroy_func)                        \
-    cls::~cls() {                                                              \
-        if (initialized())                                                     \
-            destroy_func(device(), handle(), NULL);                            \
+#define NON_DISPATCHABLE_HANDLE_DTOR(cls, destroy_func)                                                                            \
+    cls::~cls() {                                                                                                                  \
+        if (initialized())                                                                                                         \
+            destroy_func(device(), handle(), NULL);                                                                                \
     }
 
 #define STRINGIFY(x) #x
-#define EXPECT(expr)                                                           \
-    ((expr) ? true : expect_failure(STRINGIFY(expr), __FILE__, __LINE__,       \
-                                    __FUNCTION__))
+#define EXPECT(expr) ((expr) ? true : expect_failure(STRINGIFY(expr), __FILE__, __LINE__, __FUNCTION__))
 
 vk_testing::ErrorCallback error_callback;
 
-bool expect_failure(const char *expr, const char *file, unsigned int line,
-                    const char *function) {
+bool expect_failure(const char *expr, const char *file, unsigned int line, const char *function) {
     if (error_callback) {
         error_callback(expr, file, line, function);
     } else {
-        std::cerr << file << ":" << line << ": " << function
-                  << ": Expectation `" << expr << "' failed.\n";
+        std::cerr << file << ":" << line << ": " << function << ": Expectation `" << expr << "' failed.\n";
     }
 
     return false;
 }
 
-template <class T, class S>
-std::vector<T> make_handles(const std::vector<S> &v) {
+template <class T, class S> std::vector<T> make_handles(const std::vector<S> &v) {
     std::vector<T> handles;
     handles.reserve(v.size());
-    for (typename std::vector<S>::const_iterator it = v.begin(); it != v.end();
-         it++)
+    for (typename std::vector<S>::const_iterator it = v.begin(); it != v.end(); it++)
         handles.push_back((*it)->handle());
     return handles;
 }
 
-VkMemoryAllocateInfo get_resource_alloc_info(const vk_testing::Device &dev,
-                                             const VkMemoryRequirements &reqs,
+VkMemoryAllocateInfo get_resource_alloc_info(const vk_testing::Device &dev, const VkMemoryRequirements &reqs,
                                              VkMemoryPropertyFlags mem_props) {
-    VkMemoryAllocateInfo info =
-        vk_testing::DeviceMemory::alloc_info(reqs.size, 0);
+    VkMemoryAllocateInfo info = vk_testing::DeviceMemory::alloc_info(reqs.size, 0);
     dev.phy().set_memory_type(reqs.memoryTypeBits, &info, mem_props);
 
     return info;
@@ -134,8 +125,7 @@ std::vector<VkLayerProperties> GetGlobalLayers() {
 
         if (err == VK_SUCCESS) {
             layers.reserve(layer_count);
-            err =
-                vkEnumerateInstanceLayerProperties(&layer_count, layers.data());
+            err = vkEnumerateInstanceLayerProperties(&layer_count, layers.data());
         }
     } while (err == VK_INCOMPLETE);
 
@@ -147,9 +137,7 @@ std::vector<VkLayerProperties> GetGlobalLayers() {
 /*
  * Return list of Global extensions provided by the ICD / Loader
  */
-std::vector<VkExtensionProperties> GetGlobalExtensions() {
-    return GetGlobalExtensions(NULL);
-}
+std::vector<VkExtensionProperties> GetGlobalExtensions() { return GetGlobalExtensions(NULL); }
 
 /*
  * Return list of Global extensions provided by the specified layer
@@ -163,13 +151,11 @@ std::vector<VkExtensionProperties> GetGlobalExtensions(const char *pLayerName) {
 
     do {
         ext_count = 0;
-        err = vkEnumerateInstanceExtensionProperties(pLayerName, &ext_count,
-                                                     NULL);
+        err = vkEnumerateInstanceExtensionProperties(pLayerName, &ext_count, NULL);
 
         if (err == VK_SUCCESS) {
             exts.resize(ext_count);
-            err = vkEnumerateInstanceExtensionProperties(pLayerName, &ext_count,
-                                                         exts.data());
+            err = vkEnumerateInstanceExtensionProperties(pLayerName, &ext_count, exts.data());
         }
     } while (err == VK_INCOMPLETE);
 
@@ -181,28 +167,23 @@ std::vector<VkExtensionProperties> GetGlobalExtensions(const char *pLayerName) {
 /*
  * Return list of PhysicalDevice extensions provided by the ICD / Loader
  */
-std::vector<VkExtensionProperties> PhysicalDevice::extensions() const {
-    return extensions(NULL);
-}
+std::vector<VkExtensionProperties> PhysicalDevice::extensions() const { return extensions(NULL); }
 
 /*
  * Return list of PhysicalDevice extensions provided by the specified layer
  * If pLayerName is NULL, will return extensions for ICD / loader.
  */
-std::vector<VkExtensionProperties>
-PhysicalDevice::extensions(const char *pLayerName) const {
+std::vector<VkExtensionProperties> PhysicalDevice::extensions(const char *pLayerName) const {
     std::vector<VkExtensionProperties> exts;
     VkResult err;
 
     do {
         uint32_t extCount = 0;
-        err = vkEnumerateDeviceExtensionProperties(handle(), pLayerName,
-                                                   &extCount, NULL);
+        err = vkEnumerateDeviceExtensionProperties(handle(), pLayerName, &extCount, NULL);
 
         if (err == VK_SUCCESS) {
             exts.resize(extCount);
-            err = vkEnumerateDeviceExtensionProperties(handle(), pLayerName,
-                                                       &extCount, exts.data());
+            err = vkEnumerateDeviceExtensionProperties(handle(), pLayerName, &extCount, exts.data());
         }
     } while (err == VK_INCOMPLETE);
 
@@ -211,19 +192,15 @@ PhysicalDevice::extensions(const char *pLayerName) const {
     return exts;
 }
 
-bool PhysicalDevice::set_memory_type(const uint32_t type_bits,
-                                     VkMemoryAllocateInfo *info,
-                                     const VkFlags properties,
+bool PhysicalDevice::set_memory_type(const uint32_t type_bits, VkMemoryAllocateInfo *info, const VkFlags properties,
                                      const VkFlags forbid) const {
     uint32_t type_mask = type_bits;
     // Search memtypes to find first index with those properties
     for (uint32_t i = 0; i < memory_properties_.memoryTypeCount; i++) {
         if ((type_mask & 1) == 1) {
             // Type is available, does it match user properties?
-            if ((memory_properties_.memoryTypes[i].propertyFlags &
-                 properties) == properties &&
-                (memory_properties_.memoryTypes[i].propertyFlags & forbid) ==
-                    0) {
+            if ((memory_properties_.memoryTypes[i].propertyFlags & properties) == properties &&
+                (memory_properties_.memoryTypes[i].propertyFlags & forbid) == 0) {
                 info->memoryTypeIndex = i;
                 return true;
             }
@@ -247,8 +224,7 @@ std::vector<VkLayerProperties> PhysicalDevice::layers() const {
 
         if (err == VK_SUCCESS) {
             layer_props.reserve(layer_count);
-            err = vkEnumerateDeviceLayerProperties(handle(), &layer_count,
-                                                   layer_props.data());
+            err = vkEnumerateDeviceLayerProperties(handle(), &layer_count, layer_props.data());
         }
     } while (err == VK_INCOMPLETE);
 
@@ -262,8 +238,7 @@ Device::~Device() {
         return;
 
     for (int i = 0; i < QUEUE_COUNT; i++) {
-        for (std::vector<Queue *>::iterator it = queues_[i].begin();
-             it != queues_[i].end(); it++)
+        for (std::vector<Queue *>::iterator it = queues_[i].begin(); it != queues_[i].end(); it++)
             delete *it;
         queues_[i].clear();
     }
@@ -271,11 +246,9 @@ Device::~Device() {
     vkDestroyDevice(handle(), NULL);
 }
 
-void Device::init(std::vector<const char *> &layers,
-                  std::vector<const char *> &extensions) {
+void Device::init(std::vector<const char *> &extensions, VkPhysicalDeviceFeatures *features) {
     // request all queues
-    const std::vector<VkQueueFamilyProperties> queue_props =
-        phy_.queue_properties();
+    const std::vector<VkQueueFamilyProperties> queue_props = phy_.queue_properties();
     std::vector<VkDeviceQueueCreateInfo> queue_info;
     queue_info.reserve(queue_props.size());
 
@@ -302,14 +275,19 @@ void Device::init(std::vector<const char *> &layers,
     dev_info.pNext = NULL;
     dev_info.queueCreateInfoCount = queue_info.size();
     dev_info.pQueueCreateInfos = queue_info.data();
-    dev_info.enabledLayerCount = layers.size();
-    dev_info.ppEnabledLayerNames = layers.data();
+    dev_info.enabledLayerCount = 0;
+    dev_info.ppEnabledLayerNames = NULL;
     dev_info.enabledExtensionCount = extensions.size();
     dev_info.ppEnabledExtensionNames = extensions.data();
 
-    // request all supportable features enabled
-    auto features = phy().features();
-    dev_info.pEnabledFeatures = &features;
+    VkPhysicalDeviceFeatures all_features;
+    if (features) {
+        dev_info.pEnabledFeatures = features;
+    } else {
+        // request all supportable features enabled
+        all_features = phy().features();
+        dev_info.pEnabledFeatures = &all_features;
+    }
 
     init(dev_info);
 }
@@ -328,15 +306,12 @@ void Device::init_queues() {
     uint32_t queue_node_count;
 
     // Call with NULL data to get count
-    vkGetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count,
-                                             NULL);
+    vkGetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count, NULL);
     EXPECT(queue_node_count >= 1);
 
-    VkQueueFamilyProperties *queue_props =
-        new VkQueueFamilyProperties[queue_node_count];
+    VkQueueFamilyProperties *queue_props = new VkQueueFamilyProperties[queue_node_count];
 
-    vkGetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count,
-                                             queue_props);
+    vkGetPhysicalDeviceQueueFamilyProperties(phy_.handle(), &queue_node_count, queue_props);
 
     for (uint32_t i = 0; i < queue_node_count; i++) {
         VkQueue queue;
@@ -371,14 +346,12 @@ void Device::init_formats() {
         const VkFormatProperties props = format_properties(fmt);
 
         if (props.linearTilingFeatures) {
-            const Format tmp = {fmt, VK_IMAGE_TILING_LINEAR,
-                                props.linearTilingFeatures};
+            const Format tmp = {fmt, VK_IMAGE_TILING_LINEAR, props.linearTilingFeatures};
             formats_.push_back(tmp);
         }
 
         if (props.optimalTilingFeatures) {
-            const Format tmp = {fmt, VK_IMAGE_TILING_OPTIMAL,
-                                props.optimalTilingFeatures};
+            const Format tmp = {fmt, VK_IMAGE_TILING_OPTIMAL, props.optimalTilingFeatures};
             formats_.push_back(tmp);
         }
     }
@@ -395,27 +368,21 @@ VkFormatProperties Device::format_properties(VkFormat format) {
 
 void Device::wait() { EXPECT(vkDeviceWaitIdle(handle()) == VK_SUCCESS); }
 
-VkResult Device::wait(const std::vector<const Fence *> &fences, bool wait_all,
-                      uint64_t timeout) {
+VkResult Device::wait(const std::vector<const Fence *> &fences, bool wait_all, uint64_t timeout) {
     const std::vector<VkFence> fence_handles = make_handles<VkFence>(fences);
-    VkResult err = vkWaitForFences(handle(), fence_handles.size(),
-                                   fence_handles.data(), wait_all, timeout);
+    VkResult err = vkWaitForFences(handle(), fence_handles.size(), fence_handles.data(), wait_all, timeout);
     EXPECT(err == VK_SUCCESS || err == VK_TIMEOUT);
 
     return err;
 }
 
-void Device::update_descriptor_sets(
-    const std::vector<VkWriteDescriptorSet> &writes,
-    const std::vector<VkCopyDescriptorSet> &copies) {
-    vkUpdateDescriptorSets(handle(), writes.size(), writes.data(),
-                           copies.size(), copies.data());
+void Device::update_descriptor_sets(const std::vector<VkWriteDescriptorSet> &writes,
+                                    const std::vector<VkCopyDescriptorSet> &copies) {
+    vkUpdateDescriptorSets(handle(), writes.size(), writes.data(), copies.size(), copies.data());
 }
 
-void Queue::submit(const std::vector<const CommandBuffer *> &cmds,
-                   Fence &fence) {
-    const std::vector<VkCommandBuffer> cmd_handles =
-        make_handles<VkCommandBuffer>(cmds);
+void Queue::submit(const std::vector<const CommandBuffer *> &cmds, Fence &fence) {
+    const std::vector<VkCommandBuffer> cmd_handles = make_handles<VkCommandBuffer>(cmds);
     VkSubmitInfo submit_info;
     submit_info.sType = VK_STRUCTURE_TYPE_SUBMIT_INFO;
     submit_info.pNext = NULL;
@@ -427,13 +394,10 @@ void Queue::submit(const std::vector<const CommandBuffer *> &cmds,
     submit_info.signalSemaphoreCount = 0;
     submit_info.pSignalSemaphores = NULL;
 
-    EXPECT(vkQueueSubmit(handle(), 1, &submit_info, fence.handle()) ==
-           VK_SUCCESS);
+    EXPECT(vkQueueSubmit(handle(), 1, &submit_info, fence.handle()) == VK_SUCCESS);
 }
 
-void Queue::submit(const CommandBuffer &cmd, Fence &fence) {
-    submit(std::vector<const CommandBuffer *>(1, &cmd), fence);
-}
+void Queue::submit(const CommandBuffer &cmd, Fence &fence) { submit(std::vector<const CommandBuffer *>(1, &cmd), fence); }
 
 void Queue::submit(const CommandBuffer &cmd) {
     Fence fence;
@@ -453,8 +417,7 @@ void DeviceMemory::init(const Device &dev, const VkMemoryAllocateInfo &info) {
 
 const void *DeviceMemory::map(VkFlags flags) const {
     void *data;
-    if (!EXPECT(vkMapMemory(device(), handle(), 0, VK_WHOLE_SIZE, flags,
-                            &data) == VK_SUCCESS))
+    if (!EXPECT(vkMapMemory(device(), handle(), 0, VK_WHOLE_SIZE, flags, &data) == VK_SUCCESS))
         data = NULL;
 
     return data;
@@ -462,8 +425,7 @@ const void *DeviceMemory::map(VkFlags flags) const {
 
 void *DeviceMemory::map(VkFlags flags) {
     void *data;
-    if (!EXPECT(vkMapMemory(device(), handle(), 0, VK_WHOLE_SIZE, flags,
-                            &data) == VK_SUCCESS))
+    if (!EXPECT(vkMapMemory(device(), handle(), 0, VK_WHOLE_SIZE, flags, &data) == VK_SUCCESS))
         data = NULL;
 
     return data;
@@ -473,9 +435,7 @@ void DeviceMemory::unmap() const { vkUnmapMemory(device(), handle()); }
 
 NON_DISPATCHABLE_HANDLE_DTOR(Fence, vkDestroyFence)
 
-void Fence::init(const Device &dev, const VkFenceCreateInfo &info) {
-    NON_DISPATCHABLE_HANDLE_INIT(vkCreateFence, dev, &info);
-}
+void Fence::init(const Device &dev, const VkFenceCreateInfo &info) { NON_DISPATCHABLE_HANDLE_INIT(vkCreateFence, dev, &info); }
 
 NON_DISPATCHABLE_HANDLE_DTOR(Semaphore, vkDestroySemaphore)
 
@@ -485,9 +445,7 @@ void Semaphore::init(const Device &dev, const VkSemaphoreCreateInfo &info) {
 
 NON_DISPATCHABLE_HANDLE_DTOR(Event, vkDestroyEvent)
 
-void Event::init(const Device &dev, const VkEventCreateInfo &info) {
-    NON_DISPATCHABLE_HANDLE_INIT(vkCreateEvent, dev, &info);
-}
+void Event::init(const Device &dev, const VkEventCreateInfo &info) { NON_DISPATCHABLE_HANDLE_INIT(vkCreateEvent, dev, &info); }
 
 void Event::set() { EXPECT(vkSetEvent(device(), handle()) == VK_SUCCESS); }
 
@@ -499,10 +457,8 @@ void QueryPool::init(const Device &dev, const VkQueryPoolCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vkCreateQueryPool, dev, &info);
 }
 
-VkResult QueryPool::results(uint32_t first, uint32_t count, size_t size,
-                            void *data, size_t stride) {
-    VkResult err = vkGetQueryPoolResults(device(), handle(), first, count, size,
-                                         data, stride, 0);
+VkResult QueryPool::results(uint32_t first, uint32_t count, size_t size, void *data, size_t stride) {
+    VkResult err = vkGetQueryPoolResults(device(), handle(), first, count, size, data, stride, 0);
     EXPECT(err == VK_SUCCESS || err == VK_NOT_READY);
 
     return err;
@@ -510,12 +466,10 @@ VkResult QueryPool::results(uint32_t first, uint32_t count, size_t size,
 
 NON_DISPATCHABLE_HANDLE_DTOR(Buffer, vkDestroyBuffer)
 
-void Buffer::init(const Device &dev, const VkBufferCreateInfo &info,
-                  VkMemoryPropertyFlags mem_props) {
+void Buffer::init(const Device &dev, const VkBufferCreateInfo &info, VkMemoryPropertyFlags mem_props) {
     init_no_mem(dev, info);
 
-    internal_mem_.init(
-        dev, get_resource_alloc_info(dev, memory_requirements(), mem_props));
+    internal_mem_.init(dev, get_resource_alloc_info(dev, memory_requirements(), mem_props));
     bind_memory(internal_mem_, 0);
 }
 
@@ -533,8 +487,7 @@ VkMemoryRequirements Buffer::memory_requirements() const {
 }
 
 void Buffer::bind_memory(const DeviceMemory &mem, VkDeviceSize mem_offset) {
-    EXPECT(vkBindBufferMemory(device(), handle(), mem.handle(), mem_offset) ==
-           VK_SUCCESS);
+    EXPECT(vkBindBufferMemory(device(), handle(), mem.handle(), mem_offset) == VK_SUCCESS);
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(BufferView, vkDestroyBufferView)
@@ -545,13 +498,11 @@ void BufferView::init(const Device &dev, const VkBufferViewCreateInfo &info) {
 
 NON_DISPATCHABLE_HANDLE_DTOR(Image, vkDestroyImage)
 
-void Image::init(const Device &dev, const VkImageCreateInfo &info,
-                 VkMemoryPropertyFlags mem_props) {
+void Image::init(const Device &dev, const VkImageCreateInfo &info, VkMemoryPropertyFlags mem_props) {
     init_no_mem(dev, info);
 
     if (initialized()) {
-        internal_mem_.init(
-                    dev, get_resource_alloc_info(dev, memory_requirements(), mem_props));
+        internal_mem_.init(dev, get_resource_alloc_info(dev, memory_requirements(), mem_props));
         bind_memory(internal_mem_, 0);
     }
 }
@@ -566,11 +517,8 @@ void Image::init_no_mem(const Device &dev, const VkImageCreateInfo &info) {
 void Image::init_info(const Device &dev, const VkImageCreateInfo &info) {
     create_info_ = info;
 
-    for (std::vector<Device::Format>::const_iterator it = dev.formats().begin();
-         it != dev.formats().end(); it++) {
-        if (memcmp(&it->format, &create_info_.format, sizeof(it->format)) ==
-                0 &&
-            it->tiling == create_info_.tiling) {
+    for (std::vector<Device::Format>::const_iterator it = dev.formats().begin(); it != dev.formats().end(); it++) {
+        if (memcmp(&it->format, &create_info_.format, sizeof(it->format)) == 0 && it->tiling == create_info_.tiling) {
             format_features_ = it->features;
             break;
         }
@@ -586,12 +534,10 @@ VkMemoryRequirements Image::memory_requirements() const {
 }
 
 void Image::bind_memory(const DeviceMemory &mem, VkDeviceSize mem_offset) {
-    EXPECT(vkBindImageMemory(device(), handle(), mem.handle(), mem_offset) ==
-           VK_SUCCESS);
+    EXPECT(vkBindImageMemory(device(), handle(), mem.handle(), mem_offset) == VK_SUCCESS);
 }
 
-VkSubresourceLayout
-Image::subresource_layout(const VkImageSubresource &subres) const {
+VkSubresourceLayout Image::subresource_layout(const VkImageSubresource &subres) const {
     VkSubresourceLayout data;
     size_t size = sizeof(data);
     vkGetImageSubresourceLayout(device(), handle(), &subres, &data);
@@ -601,12 +547,9 @@ Image::subresource_layout(const VkImageSubresource &subres) const {
     return data;
 }
 
-VkSubresourceLayout
-Image::subresource_layout(const VkImageSubresourceLayers &subrescopy) const {
+VkSubresourceLayout Image::subresource_layout(const VkImageSubresourceLayers &subrescopy) const {
     VkSubresourceLayout data;
-    VkImageSubresource subres =
-        subresource(subrescopy.aspectMask, subrescopy.mipLevel,
-                    subrescopy.baseArrayLayer);
+    VkImageSubresource subres = subresource(subrescopy.aspectMask, subrescopy.mipLevel, subrescopy.baseArrayLayer);
     size_t size = sizeof(data);
     vkGetImageSubresourceLayout(device(), handle(), &subres, &data);
     if (size != sizeof(data))
@@ -616,11 +559,8 @@ Image::subresource_layout(const VkImageSubresourceLayers &subrescopy) const {
 }
 
 bool Image::transparent() const {
-    return (
-        create_info_.tiling == VK_IMAGE_TILING_LINEAR &&
-        create_info_.samples == VK_SAMPLE_COUNT_1_BIT &&
-        !(create_info_.usage & (VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT |
-                                VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT)));
+    return (create_info_.tiling == VK_IMAGE_TILING_LINEAR && create_info_.samples == VK_SAMPLE_COUNT_1_BIT &&
+            !(create_info_.usage & (VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT)));
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(ImageView, vkDestroyImageView)
@@ -631,13 +571,11 @@ void ImageView::init(const Device &dev, const VkImageViewCreateInfo &info) {
 
 NON_DISPATCHABLE_HANDLE_DTOR(ShaderModule, vkDestroyShaderModule)
 
-void ShaderModule::init(const Device &dev,
-                        const VkShaderModuleCreateInfo &info) {
+void ShaderModule::init(const Device &dev, const VkShaderModuleCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vkCreateShaderModule, dev, &info);
 }
 
-VkResult ShaderModule::init_try(const Device &dev,
-                                const VkShaderModuleCreateInfo &info) {
+VkResult ShaderModule::init_try(const Device &dev, const VkShaderModuleCreateInfo &info) {
     VkShaderModule mod;
 
     VkResult err = vkCreateShaderModule(dev.handle(), &info, NULL, &mod);
@@ -649,22 +587,19 @@ VkResult ShaderModule::init_try(const Device &dev,
 
 NON_DISPATCHABLE_HANDLE_DTOR(Pipeline, vkDestroyPipeline)
 
-void Pipeline::init(const Device &dev,
-                    const VkGraphicsPipelineCreateInfo &info) {
+void Pipeline::init(const Device &dev, const VkGraphicsPipelineCreateInfo &info) {
     VkPipelineCache cache;
     VkPipelineCacheCreateInfo ci;
     memset((void *)&ci, 0, sizeof(VkPipelineCacheCreateInfo));
     ci.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
     VkResult err = vkCreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     if (err == VK_SUCCESS) {
-        NON_DISPATCHABLE_HANDLE_INIT(vkCreateGraphicsPipelines, dev, cache, 1,
-                                     &info);
+        NON_DISPATCHABLE_HANDLE_INIT(vkCreateGraphicsPipelines, dev, cache, 1, &info);
         vkDestroyPipelineCache(dev.handle(), cache, NULL);
     }
 }
 
-VkResult Pipeline::init_try(const Device &dev,
-                            const VkGraphicsPipelineCreateInfo &info) {
+VkResult Pipeline::init_try(const Device &dev, const VkGraphicsPipelineCreateInfo &info) {
     VkPipeline pipe;
     VkPipelineCache cache;
     VkPipelineCacheCreateInfo ci;
@@ -673,8 +608,7 @@ VkResult Pipeline::init_try(const Device &dev,
     VkResult err = vkCreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     EXPECT(err == VK_SUCCESS);
     if (err == VK_SUCCESS) {
-        err = vkCreateGraphicsPipelines(dev.handle(), cache, 1, &info, NULL,
-                                        &pipe);
+        err = vkCreateGraphicsPipelines(dev.handle(), cache, 1, &info, NULL, &pipe);
         if (err == VK_SUCCESS) {
             NonDispHandle::init(dev.handle(), pipe);
         }
@@ -684,27 +618,23 @@ VkResult Pipeline::init_try(const Device &dev,
     return err;
 }
 
-void Pipeline::init(const Device &dev,
-                    const VkComputePipelineCreateInfo &info) {
+void Pipeline::init(const Device &dev, const VkComputePipelineCreateInfo &info) {
     VkPipelineCache cache;
     VkPipelineCacheCreateInfo ci;
     memset((void *)&ci, 0, sizeof(VkPipelineCacheCreateInfo));
     ci.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
     VkResult err = vkCreatePipelineCache(dev.handle(), &ci, NULL, &cache);
     if (err == VK_SUCCESS) {
-        NON_DISPATCHABLE_HANDLE_INIT(vkCreateComputePipelines, dev, cache, 1,
-                                     &info);
+        NON_DISPATCHABLE_HANDLE_INIT(vkCreateComputePipelines, dev, cache, 1, &info);
         vkDestroyPipelineCache(dev.handle(), cache, NULL);
     }
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(PipelineLayout, vkDestroyPipelineLayout)
 
-void PipelineLayout::init(
-    const Device &dev, VkPipelineLayoutCreateInfo &info,
-    const std::vector<const DescriptorSetLayout *> &layouts) {
-    const std::vector<VkDescriptorSetLayout> layout_handles =
-        make_handles<VkDescriptorSetLayout>(layouts);
+void PipelineLayout::init(const Device &dev, VkPipelineLayoutCreateInfo &info,
+                          const std::vector<const DescriptorSetLayout *> &layouts) {
+    const std::vector<VkDescriptorSetLayout> layout_handles = make_handles<VkDescriptorSetLayout>(layouts);
     info.pSetLayouts = layout_handles.data();
 
     NON_DISPATCHABLE_HANDLE_INIT(vkCreatePipelineLayout, dev, &info);
@@ -718,29 +648,22 @@ void Sampler::init(const Device &dev, const VkSamplerCreateInfo &info) {
 
 NON_DISPATCHABLE_HANDLE_DTOR(DescriptorSetLayout, vkDestroyDescriptorSetLayout)
 
-void DescriptorSetLayout::init(const Device &dev,
-                               const VkDescriptorSetLayoutCreateInfo &info) {
+void DescriptorSetLayout::init(const Device &dev, const VkDescriptorSetLayoutCreateInfo &info) {
     NON_DISPATCHABLE_HANDLE_INIT(vkCreateDescriptorSetLayout, dev, &info);
 }
 
 NON_DISPATCHABLE_HANDLE_DTOR(DescriptorPool, vkDestroyDescriptorPool)
 
-void DescriptorPool::init(const Device &dev,
-                          const VkDescriptorPoolCreateInfo &info) {
-    setDynamicUsage(info.flags &
-                    VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT);
+void DescriptorPool::init(const Device &dev, const VkDescriptorPoolCreateInfo &info) {
+    setDynamicUsage(info.flags & VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT);
     NON_DISPATCHABLE_HANDLE_INIT(vkCreateDescriptorPool, dev, &info);
 }
 
-void DescriptorPool::reset() {
-    EXPECT(vkResetDescriptorPool(device(), handle(), 0) == VK_SUCCESS);
-}
+void DescriptorPool::reset() { EXPECT(vkResetDescriptorPool(device(), handle(), 0) == VK_SUCCESS); }
 
-std::vector<DescriptorSet *> DescriptorPool::alloc_sets(
-    const Device &dev,
-    const std::vector<const DescriptorSetLayout *> &layouts) {
-    const std::vector<VkDescriptorSetLayout> layout_handles =
-        make_handles<VkDescriptorSetLayout>(layouts);
+std::vector<DescriptorSet *> DescriptorPool::alloc_sets(const Device &dev,
+                                                        const std::vector<const DescriptorSetLayout *> &layouts) {
+    const std::vector<VkDescriptorSetLayout> layout_handles = make_handles<VkDescriptorSetLayout>(layouts);
 
     std::vector<VkDescriptorSet> set_handles;
     set_handles.resize(layout_handles.size());
@@ -750,13 +673,11 @@ std::vector<DescriptorSet *> DescriptorPool::alloc_sets(
     alloc_info.descriptorSetCount = layout_handles.size();
     alloc_info.descriptorPool = handle();
     alloc_info.pSetLayouts = layout_handles.data();
-    VkResult err =
-        vkAllocateDescriptorSets(device(), &alloc_info, set_handles.data());
+    VkResult err = vkAllocateDescriptorSets(device(), &alloc_info, set_handles.data());
     EXPECT(err == VK_SUCCESS);
 
     std::vector<DescriptorSet *> sets;
-    for (std::vector<VkDescriptorSet>::const_iterator it = set_handles.begin();
-         it != set_handles.end(); it++) {
+    for (std::vector<VkDescriptorSet>::const_iterator it = set_handles.begin(); it != set_handles.end(); it++) {
         // do descriptor sets need memories bound?
         DescriptorSet *descriptorSet = new DescriptorSet(dev, this, *it);
         sets.push_back(descriptorSet);
@@ -764,15 +685,11 @@ std::vector<DescriptorSet *> DescriptorPool::alloc_sets(
     return sets;
 }
 
-std::vector<DescriptorSet *>
-DescriptorPool::alloc_sets(const Device &dev, const DescriptorSetLayout &layout,
-                           uint32_t count) {
-    return alloc_sets(dev,
-                      std::vector<const DescriptorSetLayout *>(count, &layout));
+std::vector<DescriptorSet *> DescriptorPool::alloc_sets(const Device &dev, const DescriptorSetLayout &layout, uint32_t count) {
+    return alloc_sets(dev, std::vector<const DescriptorSetLayout *>(count, &layout));
 }
 
-DescriptorSet *DescriptorPool::alloc_sets(const Device &dev,
-                                          const DescriptorSetLayout &layout) {
+DescriptorSet *DescriptorPool::alloc_sets(const Device &dev, const DescriptorSetLayout &layout) {
     std::vector<DescriptorSet *> set = alloc_sets(dev, layout, 1);
     return (set.empty()) ? NULL : set[0];
 }
@@ -782,8 +699,7 @@ DescriptorSet::~DescriptorSet() {
         // Only call vkFree* on sets allocated from pool with usage *_DYNAMIC
         if (containing_pool_->getDynamicUsage()) {
             VkDescriptorSet sets[1] = {handle()};
-            EXPECT(vkFreeDescriptorSets(device(), containing_pool_->GetObj(), 1,
-                                        sets) == VK_SUCCESS);
+            EXPECT(vkFreeDescriptorSets(device(), containing_pool_->GetObj(), 1, sets) == VK_SUCCESS);
         }
     }
 }
@@ -801,24 +717,20 @@ CommandBuffer::~CommandBuffer() {
     }
 }
 
-void CommandBuffer::init(const Device &dev,
-                         const VkCommandBufferAllocateInfo &info) {
+void CommandBuffer::init(const Device &dev, const VkCommandBufferAllocateInfo &info) {
     VkCommandBuffer cmd;
 
     // Make sure commandPool is set
     assert(info.commandPool);
 
-    if (EXPECT(vkAllocateCommandBuffers(dev.handle(), &info, &cmd) ==
-               VK_SUCCESS)) {
+    if (EXPECT(vkAllocateCommandBuffers(dev.handle(), &info, &cmd) == VK_SUCCESS)) {
         Handle::init(cmd);
         dev_handle_ = dev.handle();
         cmd_pool_ = info.commandPool;
     }
 }
 
-void CommandBuffer::begin(const VkCommandBufferBeginInfo *info) {
-    EXPECT(vkBeginCommandBuffer(handle(), info) == VK_SUCCESS);
-}
+void CommandBuffer::begin(const VkCommandBufferBeginInfo *info) { EXPECT(vkBeginCommandBuffer(handle(), info) == VK_SUCCESS); }
 
 void CommandBuffer::begin() {
     VkCommandBufferBeginInfo info = {};
@@ -838,12 +750,8 @@ void CommandBuffer::begin() {
     begin(&info);
 }
 
-void CommandBuffer::end() {
-    EXPECT(vkEndCommandBuffer(handle()) == VK_SUCCESS);
-}
+void CommandBuffer::end() { EXPECT(vkEndCommandBuffer(handle()) == VK_SUCCESS); }
 
-void CommandBuffer::reset(VkCommandBufferResetFlags flags) {
-    EXPECT(vkResetCommandBuffer(handle(), flags) == VK_SUCCESS);
-}
+void CommandBuffer::reset(VkCommandBufferResetFlags flags) { EXPECT(vkResetCommandBuffer(handle(), flags) == VK_SUCCESS); }
 
 }; // namespace vk_testing
